limit = 10;

$(document).ready(async function () {
    await EventUnitTemplatesModule.load_templates();
    EventUnitTemplatesModule.setup_pagination();
    await EventUnitTemplatesModule.load_eu_template_list();
});

EventUnitTemplatesApi = (function () {
    let requests = [];
    let run_all = async function (successCallback, failureCallback) {
        await Promise.all(requests.map(req => $.ajax(req))).then(successCallback, failureCallback);
        requests = [];
    };
    let add_position_to_template = function (position_template_entry) {
        requests.push({
            url: '/api/events/units/templates/' + position_template_entry.template_id + '/positions/' + position_template_entry.position_entity_id,
            type: 'POST',
            contentType: 'application/json',
            data: JSON.stringify(position_template_entry),
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte neue Position nicht speichern!");
            }
        });
    };
    let update_position_for_template = function (position_template_entry) {
        requests.push({
            url: '/api/events/units/templates/' + position_template_entry.template_id + '/positions/' + position_template_entry.position_entity_id + '/' + position_template_entry.position_template_id,
            type: 'PUT',
            contentType: 'application/json',
            data: JSON.stringify(position_template_entry),
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte geänderte Position nicht speichern!");
            }
        });
    };
    let remove_position_from_template = function (position_template_id, template_id, position_id) {
        requests.push({
            url: '/api/events/units/templates/' + template_id + '/positions/' + position_id + '/' + position_template_id,
            type: 'DELETE',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte Position nicht von der Vorlage entfernen!");
            }
        });
    };
    let update_template_core_data = function (template) {
        requests.push({
            url: '/api/events/units/templates/' + template.entity_id,
            type: 'PUT',
            contentType: 'application/json',
            data: JSON.stringify(template),
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte Namen & Beschreibung des Templates nicht speichern!");
            }
        });
    };
    let add_vehicle_position = function (vehicle_position_entry) {
        requests.push({
            url: '/api/events/units/vehicle_positions',
            type: 'POST',
            contentType: 'application/json',
            data: JSON.stringify(vehicle_position_entry),
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte neue Fahrzeugposition nicht speichern!");
            }
        });
    };
    let remove_vehicle_position = function (vehicle_position_id) {
        requests.push({
            url: '/api/events/units/vehicle_positions/' + vehicle_position_id,
            type: 'DELETE',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte Position nicht von der Vorlage entfernen!");
            }
        });
    };
    let update_vehicle_position = function (vehicle_position_entry) {
        requests.push({
            url: '/api/events/units/vehicle_positions/' + vehicle_position_entry.entity_id,
            type: 'PUT',
            contentType: 'application/json',
            data: JSON.stringify(vehicle_position_entry),
            success: function (data) {
                if (is_ok(data)) {
                }
            },
            timeout: 3000,
            error: function () {
                EventUnitTemplatesModule.show_alert("danger", "Konnte geänderte Position nicht speichern!");
            }
        });
    };
    return {
        add_position_to_template,
        update_position_for_template,
        remove_position_from_template,
        update_template_core_data,
        add_vehicle_position,
        remove_vehicle_position,
        update_vehicle_position,
        run_all
    }
}());

EventUnitTemplatesModule = (function () {
    let templates = {};
    let eu_templates = new Map();
    let loaded_template = {};
    let old_offset = 0;
    let searchbar_counter = 0;
    let pag = null;
    let all_checked = false;
    let template_modified = false;
    let activate_modified = function () {
        if (!template_modified) { //Only execute on first modification
            template_modified = true;
            $(".save-button").off("click").on("click", save).show(); //Show save button
        }
        $(this).addClass("modified");
    };
    let deactivate_modified = function () {
        if (template_modified) { //Only execute on first modification
            template_modified = false;
            $(".save-button").hide(); //Show save button
        }
    };
    let show_alert = function (level, msg) {
        let alerto = {};
        alerto.level = level;
        alerto.content = msg;
        $(".alertbox").append(templates.alert(alerto));
        $(".alert").delay(4000).slideUp(500, function () {
            $(this).alert('close');
        });
    };
    let delete_templates = function () {
        let template_list = [];
        $(".eu_template_checkbox").each(function (k, v) {
            if ($(v).prop("checked")) {
                template_list.push($(v).data("entity-id"));
            }
        });

        $.ajax({
            url: '/api/events/units/templates/',
            type: 'DELETE',
            data: JSON.stringify(template_list),
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    $(".eu_template_checkbox").each(function (k, v) {
                        if (template_list.includes($(v).data("entity-id"))) {
                            $(this).parent().parent().remove();
                        }
                    })
                }

            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let create_template = function () {
        let pdata = {};
        if (!$("#template_new_name").val()) {
            alert("Bitte alle Felder ausfüllen!");
            return
        } else {
            pdata.name = $("#template_new_name").val();
        }

        if ($("#template_new_description").val().length > 0) {
            pdata.description = $("#template_new_description").val();
        }

        $.ajax({
            url: '/api/events/units/templates',
            type: 'POST',
            contentType: 'application/json',
            data: JSON.stringify(pdata),
            success: function (data) {
                if (is_ok(data)) {
                    location.reload();
                }
            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };

    let mod_positions = (function () {
        let save = function () {

            // Save new positions
            $(".position_row.new_row").each(function () {
                let position_template_entry = {};
                position_template_entry.template_id = loaded_template.entity_id || undefined;
                position_template_entry.position_entity_id = $(this).find(".eo-search-selected").data("entity-id") || undefined;
                position_template_entry.tag = $(this).find(".add_position_tag").val() || undefined;
                position_template_entry.num = $(this).find(".add_position_num").val() || undefined;

                if (!position_template_entry.position_entity_id || !position_template_entry.num) { //Skip row if no num or position
                    return false;
                }

                EventUnitTemplatesApi.add_position_to_template(position_template_entry);
            })

            // Save modified positions
            // Using .position_row .modified (with whitespace) to select all position_rows where any child has .modified
            $(".position_row .modified").each(function () {
                let row = $(this).closest(".position_row");

                if (row.length === 0 || row.hasClass("marked-for-delete") || row.hasClass("new_row") || row.hasClass("empty_row")) { //Skip row if also marked for deletion or is new (don't edit rows which going to be deleted)
                    return false;
                }

                let position_template_entry = {};
                position_template_entry.position_template_id = row.data("position-template-id") || undefined;
                position_template_entry.template_id = loaded_template.entity_id || undefined;
                position_template_entry.position_entity_id = row.data("position-id") || undefined;
                position_template_entry.tag = row.find(".position_tag").val() || undefined;
                position_template_entry.num = row.find(".position_num").val() || undefined;
                EventUnitTemplatesApi.update_position_for_template(position_template_entry);
            })

            // Save deleted positions
            // Using .position_row.marked-for-delete to select all position_rows which also have .marked-for-delete
            $(".position_row.marked-for-delete").each(function () {
                EventUnitTemplatesApi.remove_position_from_template($(this).data("position-template-id"), loaded_template.entity_id, $(this).data("position-id"));
            })
        };
        let mark_for_delete = function () {
            let row = $(this).closest(".position_row");
            row.addClass("marked-for-delete");
            $(this).hide();
            row.find(".unremove_position_btn").show();
        };
        let unmark_for_delete = function () {
            let row = $(this).closest(".position_row");
            row.removeClass("marked-for-delete");
            $(this).hide();
            row.find(".remove_position_btn").show();
        }
        return {
            save,
            mark_for_delete,
            unmark_for_delete,
        }
    }());
    let mod_vehicle_positions = (function () {
        let save = function () {
            // Save new vehicle positions
            $(".vehicle_position_row.new_row").each(function () {
                let vehicle_position_entry = {};
                vehicle_position_entry.template_id = loaded_template.entity_id || undefined;
                vehicle_position_entry.name = $(this).find(".add_vehicle_position_name").val() || undefined;
                vehicle_position_entry.description = $(this).find(".add_vehicle_position_description").val() || undefined;
                vehicle_position_entry.required_vehicle_category = $(this).find(".add_vehicle_position_category").val() || undefined;

                if (!vehicle_position_entry.template_id || !vehicle_position_entry.name || !vehicle_position_entry.required_vehicle_category) { //Skip row if no num or position
                    return false;
                }

                EventUnitTemplatesApi.add_vehicle_position(vehicle_position_entry);
            });

            // Save modified vehicle positions
            // Using .vehicle_position_row .modified (with whitespace) to select all vehicle_position_rows where any child has .modified
            $(".vehicle_position_row .modified").each(function () {
                let row = $(this).closest(".vehicle_position_row");

                if (row.length === 0 || row.hasClass("marked-for-delete") || row.hasClass("new_row") || row.hasClass("empty_row")) { //Skip row if also marked for deletion or is new (don't edit rows which going to be deleted)
                    return false;
                }

                let vehicle_position_entry = {};
                vehicle_position_entry.entity_id = row.data("vehicle-position-id") || undefined;
                vehicle_position_entry.template_id = loaded_template.entity_id || undefined;
                vehicle_position_entry.name = row.find(".vehicle_position_name").val() || undefined;
                vehicle_position_entry.description = row.find(".vehicle_position_description").val() || undefined;
                vehicle_position_entry.required_vehicle_category = row.find(".vehicle_position_category").val() || undefined;
                EventUnitTemplatesApi.update_vehicle_position(vehicle_position_entry);
            });

            // Save deleted vehicle positions
            // Using .vehicle_position_row.marked-for-delete to select all vehicle_position_rows which also have .marked-for-delete
            $(".vehicle_position_row.marked-for-delete").each(function () {
                EventUnitTemplatesApi.remove_vehicle_position($(this).data("vehicle-position-id"));
            })
        };
        let mark_for_delete = function () {
            let row = $(this).closest(".vehicle_position_row");
            row.addClass("marked-for-delete");
            $(this).hide();
            row.find(".unremove_vehicle_position_btn").show();
        };
        let unmark_for_delete = function () {
            let row = $(this).closest(".vehicle_position_row");
            row.removeClass("marked-for-delete");
            $(this).hide();
            row.find(".remove_vehicle_position_btn").show();
        }
        return {
            save,
            mark_for_delete,
            unmark_for_delete,
        }
    }());
    let mod_template = (function () {
        let save = function () {
            let template = {};
            template.entity_id = loaded_template.entity_id || undefined;
            template.name = $("#template_detailed_name").val() || undefined;
            template.description = $("#template_detailed_description").val() || undefined;
            if (template.name !== loaded_template.name || template.description !== loaded_template.description) {
                //Name or description changed!
                EventUnitTemplatesApi.update_template_core_data(template);
            }
        };
        return {
            save: save,
        }
    }());
    let save = async function () {
        mod_positions.save();
        mod_template.save();
        mod_vehicle_positions.save();

        EventUnitTemplatesApi.run_all(async function () {
            template_modified = false; //Reset modified variable
            $(".save-button").hide(); //Hide save button
            $(".modified").removeClass("modified"); //Remove modified style from all fields

            //TODO: save animation/confirmation
            let active_template = loaded_template.entity_id;

            await load_eu_template_list(old_offset);
            await load_detailed_template(null, active_template);
        }, function () {
            console.log("Error occurred during one or more requests");
        })
    };

    let load_templates = async function () {
        const template_row = $.get("/templates/em_event_unit_templates_template_row.hbs");
        const template_detailed = $.get("/templates/em_event_unit_templates_template_detailed.hbs");
        const template_detailed_position_row = $.get("/templates/em_event_unit_templates_template_detailed_position_row.hbs");
        const template_detailed_position_row_empty = $.get("/templates/em_event_unit_templates_template_detailed_position_row_empty.hbs");
        const template_detailed_vehicle_position_row = $.get("/templates/em_event_unit_templates_template_detailed_vehicle_position_row.hbs");
        const template_detailed_vehicle_position_row_empty = $.get("/templates/em_event_unit_templates_template_detailed_vehicle_position_row_empty.hbs");
        const search = $.get("/templates/search.hbs");
        const pagination = $.get("/templates/pagination.hbs");
        const alert = $.get("/templates/alert.hbs");

        await Promise.all([template_row, template_detailed, template_detailed_position_row, template_detailed_position_row_empty, search, pagination, alert, template_detailed_vehicle_position_row, template_detailed_vehicle_position_row_empty]).then(function (res) {
            Handlebars.registerPartial('search', Handlebars.compile(res[4]));
            Handlebars.registerPartial('em_event_unit_templates_template_detailed_position_row', Handlebars.compile(res[2]));
            Handlebars.registerPartial('em_event_unit_templates_template_detailed_position_row_empty', Handlebars.compile(res[3]));
            Handlebars.registerPartial('em_event_unit_templates_template_detailed_vehicle_position_row', Handlebars.compile(res[7]));
            Handlebars.registerPartial('em_event_unit_templates_template_detailed_vehicle_position_row_empty', Handlebars.compile(res[8]));
            templates.template_row = Handlebars.compile(res[0]);
            templates.template_detailed = Handlebars.compile(res[1]);
            templates.template_detailed_position_row_empty = Handlebars.compile(res[3]);
            templates.template_detailed_vehicle_position_row_empty = Handlebars.compile(res[8]);
            templates.pagination = Handlebars.compile(res[5]);
            templates.alert = Handlebars.compile(res[6]);
        });
    };
    let setup_pagination = function(){
        pag = new Pagination("em_eu_templates_pagination", templates.pagination, ".pag", limit, load_eu_template_list);
    }
    let load_eu_template_list = async function(offset){
        if(offset === undefined || !Number.isInteger(offset)){
            offset = 0;
        }
        old_offset = offset;

        await $.ajax({
            url: '/api/events/units/templates?limit=' + limit + '&offset=' + offset,
            type: 'GET',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    eu_templates = new Map();

                    for (let t in data.templates) {
                        eu_templates.set(data.templates[t].entity_id, data.templates[t]);
                    }
                    $("#template_list_tbody").empty();

                    for (const [key, value] of eu_templates.entries()) {
                        $("#template_list_tbody").append(templates.template_row(value));
                    }

                    pag.render(data.total_template_count, offset);
                    $(".eu_template_tr").off("click").on("click", load_detailed_template);
                    $(".templates_delete_button").on("click", delete_templates);
                    $(".template_new_submit").on("click", create_template);
                }
            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let load_detailed_template = async function (context, selected_template) {
        //TODO: add warning modal if pending changes

        deactivate_modified(); //Reset modified variable
        $(".template_detailed_card").show();

        all_checked = false;

        $(".eu_template_tr").removeClass("table-primary");
        $(this).addClass("table-primary");
        let entity_id = ((selected_template) ? selected_template : $(this).data("entity_id"));

        $(".template_detailed_card_body").remove();

        loaded_template = eu_templates.get(entity_id);

        if (loaded_template) { //Check if template found for id
            await load_positions_for_template(loaded_template);
            //await load_vehicle_positions();

            //Apply qsf handler:

            $(".qsf").off("focusin focusout").on("focusin focusout", activate_modified).keyup(function (e) {
                if (event.which === 13) {
                    activate_modified(e.target);
                }
            });

            $(".template_detailed_card").show();
        }
    };
    let load_positions_for_template = async function(eu_template) {
        let positions = await $.ajax({
            url: '/api/events/units/templates/' + eu_template.entity_id + '/positions',
            type: 'GET',
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        let vehicle_positions = await $.ajax({
            url: '/api/events/units/templates/' + eu_template.entity_id + '/vehicle_positions',
            type: 'GET',
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        let vehicle_categories = await $.ajax({
            url: '/api/resources/vehicles/categories',
            type: 'GET',
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        await Promise.all([positions, vehicle_positions, vehicle_categories]).then(function (res) {
            positions = res[0];
            vehicle_positions = res[1];
            vehicle_categories = res[2];
        });

        //const vehicle_categories =
        if (is_ok(positions) && is_ok(vehicle_positions) && is_ok(vehicle_categories)) {
            eu_template.positions = positions;
            eu_template.vehicle_positions = vehicle_positions;
            eu_template.vehicle_categories = vehicle_categories;

            $(".template_detailed_card").append(templates.template_detailed(eu_template));

            // --- Positions ---

            //Add position search
            let position_search = new MiniSearchbar("add_position_search_first", null);
            position_search.setup();

            //Add position remove / unremove buttons
            $(".remove_position_btn").off("click").on("click", mod_positions.mark_for_delete); //Add delete button listener
            $(".unremove_position_btn").off("click").on("click", mod_positions.unmark_for_delete); //Add undelete button listener

            //Add new empty row when existing new row is clicked
            let add_new_empty_position_row = function () {
                $(".empty_row.position_row").removeClass("empty_row").addClass("new_row"); //Change row status from empty to new
                $(".new_row").off("click focus"); //Remove this function from row

                let sid = {};
                sid.base = "add_position_search_" + searchbar_counter;
                searchbar_counter++;

                $(".template_detailed_positions_tbody").append(templates.template_detailed_position_row_empty(sid)); //Add new empty row
                let position_search = new MiniSearchbar(sid.base, null);
                position_search.setup();

                $(".remove_position_btn").off("click").on("click", mod_positions.mark_for_delete); //Add delete button listener
                $(".unremove_position_btn").off("click").on("click", mod_positions.unmark_for_delete); //Add undelete button listener
                $(".remove_position_new_row_btn").off("click").on("click", function () {
                    $(this).closest(".new_row").remove();
                })

                $(".empty_row.position_row").on("click focus", add_new_empty_position_row); //Add this function to the empty row.
                activate_modified();
            };
            $(".empty_row.position_row").on("click focus", add_new_empty_position_row);

            // --- Vehicle Positions ---

            //Add new empty row when existing new row is clicked
            let add_new_empty_vehicle_position_row = function () {
                $(".empty_row.vehicle_position_row").removeClass("empty_row").addClass("new_row"); //Change row status from empty to new
                $(".new_row").off("click focus"); //Remove this function from row

                $(".template_detailed_vehicle_positions_tbody").append(templates.template_detailed_vehicle_position_row_empty(loaded_template)); //Add new empty row

                $(".remove_vehicle_position_btn").off("click").on("click", mod_vehicle_positions.mark_for_delete); //Add delete button listener
                $(".unremove_vehicle_position_btn").off("click").on("click", mod_vehicle_positions.unmark_for_delete); //Add undelete button listener
                $(".remove_vehicle_position_new_row_btn").off("click").on("click", function () {
                    $(this).closest(".new_row").remove();
                })

                $(".empty_row.vehicle_position_row").on("click focus", add_new_empty_vehicle_position_row); //Add this function to the empty row.
                activate_modified();
            };
            $(".empty_row.vehicle_position_row").on("click focus", add_new_empty_vehicle_position_row);

            //Set vehicle category selects to current category
            $(".vehicle_position_category").each(function () {
                $(this).val($(this).data("selected-category"));
            })

            //Add position remove / unremove buttons
            $(".remove_vehicle_position_btn").off("click").on("click", mod_vehicle_positions.mark_for_delete); //Add delete button listener
            $(".unremove_vehicle_position_btn").off("click").on("click", mod_vehicle_positions.unmark_for_delete); //Add undelete button listener
        }
    };
    return {
        load_templates: load_templates,
        setup_pagination: setup_pagination,
        load_eu_template_list: load_eu_template_list,
        show_alert: show_alert,
    }
}());