let all_checked = false;
let limit = 10;
let offset = 0;
let page = 1;
let total_page_count = 0;

$( document ).ready(function() {
    EventUnitPositionsModule.load_templates();
    //Do not execute code here which relies on templates. Use EventUnitTemplateModule.start function instead
});

EventUnitPositionsModule = ( function() {
    let templates = {};
    let license_categories = [];
    let qualifications = [];
    let positions = [];

    let load_templates = function(){
        $.get("/templates/em_event_unit_templates_condition.hbs", function(res) {
            templates.condition = Handlebars.compile(res);
            start();
        });
        $.get("/templates/em_event_unit_templates_condition_line.hbs", function(res) {
            templates.condition_line = Handlebars.compile(res);
            start();
        });
        $.get("/templates/em_event_unit_templates_position_row.hbs", function(res) {
            templates.position_row = Handlebars.compile(res);
            start();
        });
        $.get("/templates/em_event_unit_templates_position_detailed.hbs", function(res) {
            templates.position_detailed = Handlebars.compile(res);
            start();
        });
        load_licenses();
        load_qualifications();
    };
    let start = function(){
        if(templates.condition_line && templates.position_detailed && templates.condition && templates.condition_line && license_categories.length > 0 && qualifications.length > 0){
            load_condition_engine();
            load_position_list();
            $(".check_all_positions").on("click", function(){
                $(".eu_position_checkbox").prop("checked", !all_checked);
                all_checked = !all_checked
            });
            $(".position_list_nav_next").on("click", pagination_forward);
            $(".position_list_nav_back").on("click", pagination_backward);
            $(".position_list_nav_numbers").on("click", pagination_numbers);
            $(".positions_delete_button").on("click", delete_positions);
        }
    };
    let load_position_list = function(){
        offset = (page-1)*limit;
        $("#position_list_tbody").html("");
        $.ajax({
            url: '/api/events/units/positions?limit='+limit+'&offset='+offset,
            type: 'GET',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    positions = data.positions;
                    console.log(positions);
                    total_page_count = Math.ceil(data.total_position_count/limit);
                    $(positions).each(function(){
                        $("#position_list_tbody").append(templates.position_row(this));
                    });
                    pagination_update();
                    $(".eu_position_tr").off("click").on("click", load_detailed_position);
                }
            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let load_condition_engine = function(){
        let con = {};
        con.operator_name = "UND";
        con.operator = "AND";

        $("#position_new_condition_container").append(templates.condition(con));
        $(".position_new_condition_card_operator").first().remove();
        $(".position_new_remove_group").first().remove();

        $(".position_new_add_group").off("click").on("click", add_new_group);
        $(".operator_selection").on("change", operator_change);
        $(".position_new_add_condition").off("click").on("click", add_new_condition_line);
        $(".position_new_submit ").on("click", function(){
            save_position(conditions_to_json("#position_new"));
        });
    };
    let add_new_group = function(){
        let selected_operator = $(this).parent().parent().children(".card-header").find(".operator_selection_select option:selected");
        let con = {};
        con.operator_name = selected_operator.text();
        con.operator = selected_operator.val();

        $(this).parent().parent().find(".position_new_condition_card_main_body").first().append(templates.condition(con))
        $(".position_new_add_group").off("click").on("click", add_new_group);
        $(".operator_selection").on("change", operator_change);
        $(".position_new_add_condition").off("click").on("click", add_new_condition_line);
        $(".position_new_remove_group").off("click").on("click", remove_group);
    };
    let add_new_condition_line = function(){
        let con = {};
        con.operator_name = $(this).parent().parent().children(".card-header").find(".operator_selection_select option:selected").text();
        $(this).parent().parent().find(".position_new_condition_card_main_body").first().append(templates.condition_line(con));
        $(".position_new_remove_condition").on("click", remove_condition_line);
        $(".position_new_condition_type").on("change", condition_change_type);
        $(".position_new_condition_qual_category").on("change", condition_change_qual_category);
    };
    let operator_change = function(){
        //update span tag operator notice if operator changed
        $(this).closest(".position_new_condition_card").children(".card-body").children(".position_new_condition_card_main_body").children(".position_new_condition_card_operator").html($(this).find("option:selected").text());
        //update data-operator in .position_condition
        $(this).closest(".position_condition").attr("data-operator", $(this).find("option:selected").val());
        $(this).closest(".position_condition").data("operator", $(this).find("option:selected").val());
    };
    let remove_condition_line = function(){
        $(this).parent().prev().remove();
        $(this).parent().remove();
    };
    let remove_group = function(){
        console.log("called remove_grup");
        $(this).closest(".position_condition").prev().remove();
        $(this).closest(".position_condition").remove();
    };
    let load_licenses = function(){
        $.ajax({
            url: '/api/members/license_categories',
            type: 'GET',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    license_categories = data;
                    start();
                }
            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let load_qualifications = function(){
        $.ajax({
            url: '/api/members/qualifications',
            type: 'GET',
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    qualifications = data;
                    start();
                }

            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let condition_change_type = function(){
        let parent = $(this).parent();
        parent.find(".position_new_condition_qual_category").html("<option value='null'></option>");
        parent.find(".position_new_condition_value").html("<option value='null'></option>");
        if($(this).val() === "qualification"){
            $(qualifications).each(function(){
                parent.find(".position_new_condition_qual_category").append("<option value=\""+this.id+"\">"+this.name+"</option>")
            });
            parent.find(".position_new_condition_qual_category").show();
        }else if($(this).val() === "licenses"){
            parent.find(".position_new_condition_qual_category").hide();
            $(license_categories).each(function(){
                parent.find(".position_new_condition_value").append("<option value='"+this.name+"'>"+this.name+"</option>")
            });
        }else{
            parent.find(".position_new_condition_qual_category").hide();
        }
    };
    let condition_change_qual_category = function(){
        let valfield = $(this).parent().find(".position_new_condition_value");
        valfield.html("<option value='null'></option>");
        for(let i = 0; i<qualifications.length;i++){
            if($(this).val() === qualifications[i].id){
                $(qualifications[i].qualifications).each(function(){
                   valfield.append("<option value=\""+this.id+"\">"+this.name+"</option>")
                });
                break;
            }
        }
    };
    let conditions_from_json = function(conditions, destination){
        let parse_error = false;

        let parse_line = function(line, operator_name){
            let parameters = {};
            parameters.operator_name = operator_name;

            let type = Object.keys(line)[0];
            if(type === "license"){
                parameters.type = "licenses"
                parameters.value = line[type];
            }else if(type === "qualification"){
                parameters.type = "qualification"
                parameters.value = line[type];
            }else{
                parse_error = true;
            }
            return templates.condition_line(parameters)
        };
        let parse_group = function(group){
            let parameters = {};
            parameters.conditions = [];

            for(let i=0;i<group.length;i++){
                if(i === 0){
                    if(group[0] !== "AND" && group[0] !== "OR"){
                        parse_error = true;
                        return;
                    }else{

                        parameters.operator = group[0];
                        if(parameters.operator === "AND"){
                            parameters.operator_name = "UND";
                        }else{
                            parameters.operator_name = "ODER";
                        }
                    }
                }else{
                    if(Array.isArray(group[i])){
                        $(group[i]).each(function(){
                            parameters.condition_group = parse_group(group[i]);
                        })
                    }else{
                        parameters.conditions.push(parse_line(group[i], parameters.operator_name));
                    }
                }
            }
            return templates.condition(parameters)
        };
        if($.isEmptyObject(conditions.conditions)){
            destination.append(templates.condition({operator: "AND", operator_name: "UND"}))
        }else{
            destination.append(parse_group(conditions.conditions));
        }

        //Apply operators
        destination.find(".operator_selection_select").each(function(){
            $(this).val($(this).data("operator")).change();
        });
        //Apply condition types
        destination.find(".position_new_condition_type").each(function(){
            $(this).val($(this).data("condition-type")).change();
        });
        //Apply values
        destination.find(".position_new_condition_value").each(function(){
            let type = $(this).data("condition-type");
            let val = $(this).data("value");
            let select = $(this);

            if(type === "licenses"){
                $(license_categories).each(function(){
                   if(this.name === val){
                        select.append("<option value='"+this.name+"' selected>"+this.name+"</option>");
                   }else{
                        select.append("<option value='"+this.name+"'>"+this.name+"</option>");
                   }
                });
            }else if(type === 'qualification'){
                let cat_select = select.parent().find(".position_new_condition_qual_category");
                cat_select.show();
                let qual_category = "";

                $(qualifications).each(function(){
                    let cat = this;

                    $(this.qualifications).each(function(){
                       if(this.id === val){
                           qual_category = cat;
                           select.append("<option value='"+this.id+"' selected>"+this.name+"</option>")
                       }else{
                           select.append("<option value='"+this.id+"'>"+this.name+"</option>")
                       }
                    });

                    if(cat === qual_category){
                        cat_select.append("<option value='"+cat.id+"' selected>"+cat.name+"</option>")
                    }else{
                        cat_select.append("<option value='"+cat.id+"'>"+cat.name+"</option>")
                    }

                });

            }
        });
        $(".position_new_add_group").off("click").on("click", add_new_group);
        $(".operator_selection").on("change", operator_change);
        $(".position_new_add_condition").off("click").on("click", add_new_condition_line);
        $(".position_new_condition_card_operator").first().remove();
        $(".position_detailed_submit ").on("click", function(){
            update_position(conditions_to_json("#position_detailed"));
        });
        $(".position_new_remove_condition").on("click", remove_condition_line);
        $(".position_new_condition_type").on("change", condition_change_type);
        $(".position_new_condition_qual_category").on("change", condition_change_qual_category);
        $(".position_new_remove_group").off("click").on("click", remove_group);


    }
    let conditions_to_json = function(dest){
        let json = {};
        let parse_error = false;

        let parse_line = function(line){
            let type = $(line).find(".position_new_condition_type").val();

            if(type === "licenses"){
                let licensev = $(line).find(".position_new_condition_value").val();
                if(licensev === "null"){
                    console.log("parse error: license is null.");
                    parse_error = true;
                }else{
                    return {license: licensev}
                }
            }else if(type === "qualification"){
                let qualv = $(line).find(".position_new_condition_value").val();

                if(qualv === "null"){
                    console.log("parse error: qual is null.");
                    parse_error = true;
                }else{
                    return{qualification: qualv}
                }
            }else{
                parse_error = true;
            }
        }
        let parse_condition_group = function(condition){
            let jresult = [];
            let operation = $(condition).data("operator");
            jresult.push(operation);
            let condition_lines = $(condition).find(".position_new_condition_card_main_body").first().children(".position_new_condition_line");
            $(condition_lines).each(function(){
                if(!parse_error) {
                    jresult.push(parse_line(this));
                }
            });
            let condition_groups = $(condition).find(".position_new_condition_card_main_body").first().children(".position_condition");
            $(condition_groups).each(function(){
                if(!parse_error){
                    jresult.push(parse_condition_group(this));
                }
            });
            return jresult;
        }

        $(dest+"_condition_container").children(".position_condition").each(function(){
            json.conditions = parse_condition_group(this);
        });

        let result = {};
        if($(dest+"_name").val()){
            result.name = $(dest+"_name").val();
        }else{
            console.log("parse error: name missing");
            parse_error=true;
        }
        if($(dest+"_description").val()){
            result.description = $(dest+"_description").val();
        }
        result.entity_id = $(dest+"_entity_id").val();
        result.requirements = JSON.stringify(json);

        if(parse_error){
            alert("Bitte alle Felder ausfüllen und ggf. leere Felder entfernen.")
        }else{
            return result;
        }
    };
    let save_position = function(pdata){
        $.ajax({
            url: '/api/events/units/positions',
            type: 'POST',
            contentType: 'application/json',
            data: JSON.stringify(pdata),
            success: function (data) {
                if (is_ok(data)) {
                    location.reload();
                }

            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let pagination_forward = function(){
        if(page < total_page_count){
            page++;
            load_position_list();
        }
    };
    let pagination_backward = function(){
        if(page > 1){
            page--;
            load_position_list();
        }
    };
    let pagination_numbers = function(){
        page = parseInt($(this).text());
        load_position_list();
    };
    let pagination_update = function(){
        $(".position_list_nav_numbers").remove();
        for(let i = total_page_count; i>0;i--){
            if(page === i){
                $(".position_list_nav_back_li").after("<li class=\"page-item position_list_nav_numbers disabled\"><a class=\"page-link\" href=\"#\">"+i+"</a></li>");
            }else {
                $(".position_list_nav_back_li").after("<li class=\"page-item position_list_nav_numbers\"><a class=\"page-link\" href=\"#\">" + i + "</a></li>");
            }
        }
        $(".position_list_nav_numbers").on("click", pagination_numbers);

        if(page === total_page_count){
            $(".position_list_nav_next").parent().addClass("disabled");
        }else{
            $(".position_list_nav_next").parent().removeClass("disabled");
        }
        if(page === 1){
            $(".position_list_nav_back").parent().addClass("disabled");
        }else{
            $(".position_list_nav_back").parent().removeClass("disabled");
        }
    };
    let delete_positions = function(){
        let position_list = [];
        $(".eu_position_checkbox").each(function(k, v){
            if($(v).prop("checked")){
                position_list.push($(v).data("entity-id"));
            }
        });

        $.ajax({
            url: '/api/events/units/positions/',
            type: 'DELETE',
            data: JSON.stringify(position_list),
            contentType: 'application/json',
            success: function (data) {
                if (is_ok(data)) {
                    $(".eu_position_checkbox").each(function(k,v){
                        if(position_list.includes($(v).data("entity-id"))){
                            $(this).parent().parent().remove();
                        }
                    })
                }

            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    };
    let load_detailed_position = function(){
        $(".position_detailed_card").show();

        $(".eu_position_tr").removeClass("table-primary");
        $(this).addClass("table-primary");
        let entity_id = $(this).data("entity_id");

        $(".position_detailed_card_body").remove();
        $(positions).each(function(){
            if(this.entity_id === entity_id){
                $(".position_detailed_card").append(templates.position_detailed(this));
                if(this.requirements){
                    conditions_from_json(this.requirements, $("#position_detailed_condition_container"))
                }
                return false;
            }
        });
        //$(".organiser_detailed_submit").unbind().on("click", OrganiserModule.update_organiser);
        $(".position_detailed_card").show();
    }
    let update_position = function(position){
        $.ajax({
            url: '/api/events/units/positions/'+position.entity_id,
            type: 'PUT',
            contentType: 'application/json',
            data: JSON.stringify(position),
            success: function (data) {
                if (is_ok(data)) {
                    location.reload();
                }
            },
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
    }
    return{
        load_templates: load_templates,
        start: start,
    };
}());