let entries_per_page = 10;
let page = 1;
let all_checked = false;
let total_page_count = 1;

$( document ).ready(function() {
    OrganiserModule.load_templates();
    OrganiserModule.load_organisers();
    $(".organiser_new_submit").on("click", OrganiserModule.save_new_organiser);

    $(".check_all_organisers").on("click", function(){
        if(all_checked){
            $(".organiser_checkbox").attr("checked", false);
            all_checked = false;
        }else{
            $(".organiser_checkbox").attr("checked", true);
            all_checked = true;
        }

    });
    $(".organisers_delete_button").on("click", OrganiserModule.remove_organisers);
    $(".organiser_list_nav_next").on("click", OrganiserModule.pagination_forward);
    $(".organiser_list_nav_back").on("click", OrganiserModule.pagination_backward);
    $(".organiser_list_nav_numbers").on("click", OrganiserModule.pagination_numbers);
});

OrganiserModule = ( function() {
    let templates = {};
    let organisers = [];
    let pagination_forward = function(){
        if(page < total_page_count){
            page++;
            OrganiserModule.load_organisers();
        }
    };
    let pagination_backward = function(){
        if(page > 1){
            page--;
            OrganiserModule.load_organisers();
        }
    };
    let pagination_numbers = function(){
        page = parseInt($(this).text());
        OrganiserModule.load_organisers();
    };
    let pagination_update = function(){
        $(".organiser_list_nav_numbers").remove();
        for(let i = total_page_count; i>0;i--){
            if(page === i){
                $(".organiser_list_nav_back_li").after("<li class=\"page-item organiser_list_nav_numbers disabled\"><a class=\"page-link\" href=\"#\">"+i+"</a></li>");
            }else {
                $(".organiser_list_nav_back_li").after("<li class=\"page-item organiser_list_nav_numbers\"><a class=\"page-link\" href=\"#\">" + i + "</a></li>");
            }
        }
        $(".organiser_list_nav_numbers").on("click", OrganiserModule.pagination_numbers);

        if(page === total_page_count){
            $(".organiser_list_nav_next").parent().addClass("disabled");
        }else{
            $(".organiser_list_nav_next").parent().removeClass("disabled");
        }
        if(page === 1){
            $(".organiser_list_nav_back").parent().addClass("disabled");
        }else{
            $(".organiser_list_nav_back").parent().removeClass("disabled");
        }
    };
    let load_templates = function(){
        $.get("/templates/em_organiser_organiser_row.hbs", function( res) {
            templates.organiser_row = Handlebars.compile(res);
        });
        $.get("/templates/em_organiser_detailed.hbs", function( res) {
            templates.organiser_detailed = Handlebars.compile(res);
        });
    };
    let load_organisers = function(){
        $.ajax({
            type: "GET",
            url: "/api/event_organisers?entries=" + entries_per_page + "&page=" + page,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Verbindung zum Server unterbrochen!");
            },
            success: function (data) {
                if (is_ok(data)) {
                    $(".organiser_tr").remove();
                    let total_organiser_count = data.total_organiser_count;
                    total_page_count = Math.ceil(total_organiser_count/entries_per_page);

                    OrganiserModule.pagination_update();

                    $(data.organisers).each(function(){
                        $("#organiser_list_tbody").append(templates.organiser_row(this));
                        organisers.push(this);
                    });
                    $(".organiser_tr").unbind("click").on("click", OrganiserModule.load_details);
                }
            }
        });
    };
    let load_details = function(){
        $(".organiser_tr").removeClass("table-primary");
        $(this).addClass("table-primary");
        let entity_id = $(this).data("entity_id");

        $(".organiser_detailed_card_body").remove();
        $(organisers).each(function(){
            if(this.entity_id === entity_id){
                $(".organiser_detailed_card").append(templates.organiser_detailed(this));
                return false;
            }
        });
        $(".organiser_detailed_submit").unbind().on("click", OrganiserModule.update_organiser);
        $(".organiser_detailed_card").show();
    };
    let save_new_organiser = function(){
        let organiser = {};
        organiser.company = $("#organiser_new_company").val();
        organiser.firstname = $("#organiser_new_firstname").val();
        organiser.lastname = $("#organiser_new_lastname").val();
        organiser.phone = $("#organiser_new_phone").val();
        organiser.email = $("#organiser_new_email").val();
        organiser.other = $("#organiser_new_other").val();

        if(organiser.company !== undefined && organiser.lastname !== undefined){
            if(organiser.company.trim() == '' && organiser.lastname.trim() == ''){
                alert("Es muss zumindest Firma/Organisation oder Nachname ausgefüllt sein!");
                return
            }
        }else{
            alert("Es muss zumindest Firma/Organisation oder Nachname ausgefüllt sein!");
            return
        }

        $.ajax({
            type: "POST",
            url: "/api/event_organisers",
            contentType: 'application/json',
            timeout: 3000,
            data: JSON.stringify(organiser),
            error: function () {
                alert("Verbindung zum Server unterbrochen!");
            },
            success: function (data) {
                if (is_ok(data)) {
                    $("#organiser_list_tbody").append(templates.organiser_row(data));
                    $(".organiser_tr").unbind("click").on("click", OrganiserModule.load_details);

                    organisers.push(data);
                    $(".organiser_create_new_card :input").val("");
                }
            }
        });
    };
    let remove_organisers = function(){
        let organiser_list = [];
        $(".organiser_checkbox").each(function(k, v){
            if($(v).prop("checked")){
                organiser_list.push($(v).data("entity-id"));
            }
        });
        $.ajax({
            type: "DELETE",
            url: "/api/event_organisers/",
            contentType: 'application/json',
            timeout: 3000,
            data: JSON.stringify(organiser_list),
            error: function () {
                alert("Verbindung zum Server unterbrochen!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    $(".organiser_checkbox").each(function(k,v){
                        if(organiser_list.includes($(v).data("entity-id"))){
                            $(this).parent().parent().remove();
                        }
                    })
                }
            }
        });
    };
    let update_organiser = function(){
        let organiser = {};
        organiser.entity_id = $("#organiser_detailed_entity_id").val();
        organiser.company = $("#organiser_detailed_company").val();
        organiser.firstname = $("#organiser_detailed_firstname").val();
        organiser.lastname = $("#organiser_detailed_lastname").val();
        organiser.phone = $("#organiser_detailed_phone").val();
        organiser.email = $("#organiser_detailed_email").val();
        organiser.other = $("#organiser_detailed_other").val();

        if(organiser.company !== undefined && organiser.lastname !== undefined){
            if(organiser.company.trim() == '' && organiser.lastname.trim() == ''){
                alert("Es muss zumindest Firma/Organisation oder Nachname ausgefüllt sein!");
                return
            }
        }else{
            alert("Es muss zumindest Firma/Organisation oder Nachname ausgefüllt sein!");
            return
        }

        $.ajax({
            type: "PUT",
            url: "/api/event_organisers/" + organiser.entity_id,
            contentType: 'application/json',
            timeout: 3000,
            data: JSON.stringify(organiser),
            error: function () {
                alert("Verbindung zum Server unterbrochen!");
            },
            success: function (data) {
                if (is_ok(data)) {
                    window.location.reload(true);
                }
            }
        });
    };
    return{
        load_templates: load_templates,
        load_organisers: load_organisers,
        load_details: load_details,
        save_new_organiser: save_new_organiser,
        remove_organisers: remove_organisers,
        update_organiser: update_organiser,
        pagination_numbers: pagination_numbers,
        pagination_forward: pagination_forward,
        pagination_backward: pagination_backward,
        pagination_update: pagination_update,
    };
})();