let filter_shown = false;
let limit = 10;

$( document ).ready(async function() {
    await MemberListModule.load_templates();
    await MemberListModule.load_groups_filter();
    MemberListModule.setup_pagination();
    MemberListModule.load_members(0);

    $(".mm_list_filter_button").on("click", function(){
        if(filter_shown){
            $(".filter_card").hide();
        }else{
            $(".filter_card").show();
        }
        filter_shown = !filter_shown;
    });

    $(".mm_list_filter_submit").on("click", function(){
        MemberListModule.load_members(0)
    })
});

MemberListModule = (function(){
    let templates = {};
    let pag = {};
    let sort_field = "lastname";
    let sort_dir = "asc";
    let old_offset = 0;

    let load_templates = async function(){
            const mm_list = $.get("/templates/mm_list.hbs");
            const pagination = $.get("/templates/pagination.hbs");
            const mm_group_filter = $.get("/templates/mm_group_filter.hbs");
            await Promise.all([mm_list, pagination, mm_group_filter]).then(function(res){
                templates.mm_list = Handlebars.compile(res[0]);
                templates.pagination = Handlebars.compile(res[1]);
                templates.mm_group_filter = Handlebars.compile(res[2]);
            });
    };
    let load_groups_filter = async function(){
        let gvp = await check_for_permission_async("modules.member_management.groups.view");
        if(gvp) {
            $.ajax({
                type: "GET",
                url: "/api/groups",
                contentType: 'application/json',
                timeout: 3000,
                error: function () {
                    alert("Es ist ein Fehler aufgetreten!");
                },
                success: function (data) {
                    if (is_ok(data)) {
                        $(data.groups).each(function () {
                            if(this.name !== "default"){ //Do not show default group
                                $("#mm_list_filter_groups").append(templates.mm_group_filter(this))
                            }
                        })
                    }
                }
            });
        }else{
            $("#mm_list_filter_groups").html("<p>Keine Berechtigung Gruppen abzurufen.</p>");
        }
    };
    let sort_by_column_listener = function(){
        let new_sort_field = $(this).data("sort-name");

        //if user clicks same sort col again, just change the direction
        if(new_sort_field === sort_field){
            if(sort_dir === "asc"){
                sort_dir = "desc"
            }else{
                sort_dir = "asc"
            }
        }else{
            sort_field = new_sort_field;
            sort_dir = "asc";
        }

        load_members(old_offset)
    };
    let render_filter = function(){
        let selected_groups = "";
        $(".mm_list_group_filter_checkbox").each(function(){
            if($(this).prop("checked")){
                selected_groups += $(this).val()+","
            }
        });
        return selected_groups;
    }
    let load_members = function(offset, args){
        if(offset === undefined || !Number.isInteger(offset)){
            offset = 0;
        }
        old_offset = offset;

        if(args === undefined){
            args = "";
        }
        let selected_groups = render_filter();
        if(selected_groups !== ""){
            args += "&groups="+selected_groups
        }

        let sort = sort_field+":"+sort_dir;

        $.ajax({
            type: "GET",
            url: "/api/members/?limit=" + limit + "&offset=" + offset + "&sort="+sort+args,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            },
            success: function (data) {
                if (is_ok(data)) {
                    if(sort_field === "personnel_number" && sort_dir === "asc"){
                        data.sort_personnel_number_asc = true;
                    }
                    if(sort_field === "personnel_number" && sort_dir === "desc"){
                        data.sort_personnel_number_desc = true;
                    }
                    if(sort_field === "firstname" && sort_dir === "asc"){
                        data.sort_firstname_asc = true;
                    }
                    if(sort_field === "firstname" && sort_dir === "desc"){
                        data.sort_firstname_desc = true;
                    }
                    if(sort_field === "lastname" && sort_dir === "asc"){
                        data.sort_lastname_asc = true;
                    }
                    if(sort_field === "lastname" && sort_dir === "desc"){
                        data.sort_lastname_desc = true;
                    }
                    if(sort_field === "date_of_birth" && sort_dir === "asc"){
                        data.sort_date_of_birth_asc = true;
                    }
                    if(sort_field === "date_of_birth" && sort_dir === "desc"){
                        data.sort_date_of_birth_desc = true;
                    }

                    $(".member_list").html(templates.mm_list(data));
                    MemberDeleteModule.set_listeners();
                    $(".sort-col").off("click").on("click", sort_by_column_listener)

                    pag.render(data.count, offset);
                }
            }
        });
    };
    let setup_pagination = function(){
        pag = new Pagination("mm_search_pagination", templates.pagination, ".mm_list_footer", limit, load_members);
    };
    return{
        load_templates: load_templates,
        load_members: load_members,
        setup_pagination: setup_pagination,
        load_groups_filter: load_groups_filter,
        render_filter: render_filter,
    }
}());