active_permission = null;

$(document).ready(async function () {
    $("#settings_permissions_role").on("change", async function () {
        await RequestListModule.load_role();
    });
    await RequestListModule.load_templates();
});

RequestListModule = (function () {
    let templates = {};
    let load_templates = async function () {
        const permission_list = $.get("/templates/settings_role_permissions.hbs");
        const context_list = $.get("/templates/settings_role_permissions_context.hbs");

        await Promise.all([permission_list, context_list]).then(function (res) {
            templates.permission_list = Handlebars.compile(res[0]);
            templates.context_list = Handlebars.compile(res[1]);
        });
    };
    let permission_checkbox_toggle = async function () {
        let checkbox = $(this);
        if (checkbox.prop("checked")) {
            await add_permission_to_role($("#settings_permissions_role").val(), checkbox.data("permission"));
        } else {
            await remove_permission_from_role($("#settings_permissions_role").val(), checkbox.data("permission"));
        }
    };
    let permission_context_checkbox_toggle = async function () {
        console.log("removing/adding context")
        let checkbox = $(this);
        if (checkbox.prop("checked")) {
            await add_context($("#settings_permissions_role").val(), active_permission, checkbox.data("context-id"));
        } else {
            await remove_context($("#settings_permissions_role").val(), active_permission, checkbox.data("context-id"));
        }
    };
    let add_permission_to_role = async function (role, permission) {
        const res = $.ajax({
            type: "POST",
            url: "/api/roles/" + role + "/permissions/" + permission,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let remove_permission_from_role = async function (role, permission) {
        const res = $.ajax({
            type: "DELETE",
            url: "/api/roles/" + role + "/permissions/" + permission,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let add_context = async function (role, permission, context) {
        const res = $.ajax({
            type: "POST",
            url: "/api/roles/" + role + "/permissions/" + permission+"/context/"+context,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let remove_context = async function (role, permission, context) {
        const res = $.ajax({
            type: "DELETE",
            url: "/api/roles/" + role + "/permissions/" + permission+"/context/"+context,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let get_permissions = async function () {
        const res = $.ajax({
            type: "GET",
            url: "/api/permissions/",
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let combine_permissions = function (all_permissions, positive_permissions) {
        let res = [];

        function is_in_list(permission, list) {
            for (let i = 0; i < list.length; i++) {
                if (permission === list[i].permission) {
                    return true;
                }
            }
            return false;
        }

        $(all_permissions).each(function () {
            let permission = this.permission;

            let temp = {};
            temp.permission = permission;
            temp.description = this.description;
            temp.context = this.context;
            temp.context_type = this.context_type;

            temp.has_permission = is_in_list(permission, positive_permissions);

            if (this.context === false) {
                temp.context_combined = "ohne";
            }
            if (this.context === true) {
                temp.context_combined = this.context_type;
            }

            res.push(temp);
        });

        return res;
    };
    let load_role = async function () {
        let role = $("#settings_permissions_role").val();
        let permission_card = $(".settings_permissions_permission_card");

        if (role !== "none") {
            $.ajax({
                type: "GET",
                url: "/api/roles/" + role + "/permissions/",
                contentType: 'application/json',
                timeout: 3000,
                error: function () {
                    alert("Es ist ein Fehler aufgetreten!");
                },
                success: async function (data) {
                    if (is_ok(data)) {
                        console.log(data);

                        let res = {};

                        let all_permissions = await get_permissions();
                        res.permissions = combine_permissions(all_permissions, data);

                        console.log(res.permissions);

                        permission_card.html(templates.permission_list(res));
                        $(".permission_checkbox").off("change").on("change", RequestListModule.permission_checkbox_toggle);
                        $(".permission_row").off("click").on("click", load_context);
                    }
                }
            });
        } else {
            permission_card.html("Bitte Rolle wählen.");
        }

    };
    let load_context = async function () {
        let role = $("#settings_permissions_role").val();
        let permission = $(this).parent().find(".permission_checkbox").data("permission");
        active_permission = permission;

        let has_context = $(this).parent().data("context");
        let context_type = $(this).parent().data("context-type");

        if(has_context) {
            let res = {};

            let context = await get_context(role, permission);
            if(context_type === "groups" || context_type === "members" || context_type === "entity"){
                let tempres = [];
                let groups = await load_groups_async();
                for(let i=0;i<groups.length;i++){

                    let temp = {};
                    temp.group_id = groups[i].group_id;
                    temp.name = groups[i].name;
                    temp.description = groups[i].description;

                    temp.has_this_context = false;

                    for(let b=0;b<context.length;b++){
                        if(!temp.has_this_context){
                            if(groups[i].group_id === context[b]){
                                temp.has_this_context = true;
                                break;
                            }
                        }
                    }

                    tempres.push(temp);
                }
                res.groups = tempres;
            }
            if(context_type === "vehicles" || context_type === "entity"){
                let tempres = [];
                let vehicles = (await get_vehicles()).vehicle_list;
                for(let i=0;i<vehicles.length;i++){

                    let temp = {};
                    temp.entity_id = vehicles[i].entity_id;
                    temp.identifier = vehicles[i].identifier;
                    temp.description = vehicles[i].description;
                    temp.numberplate = vehicles[i].numberplate;

                    temp.has_this_context = false;

                    for(let b=0;b<context.length;b++){
                        if(!temp.has_this_context){
                            if(vehicles[i].entity_id === context[b]){
                                temp.has_this_context = true;
                                break;
                            }
                        }
                    }

                    tempres.push(temp);
                }
                res.vehicles = tempres;
            }
            if(context_type === "members" || context_type === "entity"){
                let tempres = [];
                let members = (await get_members()).members;
                console.log(members);
                for(let i=0;i<members.length;i++){

                    let temp = {};
                    temp.entity_id = members[i].entity_id;
                    temp.firstname = members[i].firstname;
                    temp.lastname = members[i].lastname;

                    temp.has_this_context = false;

                    for(let b=0;b<context.length;b++){
                        if(!temp.has_this_context){
                            if(members[i].entity_id === context[b]){
                                temp.has_this_context = true;
                                break;
                            }
                        }
                    }

                    tempres.push(temp);
                }
                res.members = tempres;
            }

            $(".settings_permissions_permission_context_card").html(templates.context_list(res));

            $(".permission_context_checkbox").off("click").on("click", permission_context_checkbox_toggle);
        }else{
            $(".settings_permissions_permission_context_card").html("Kein Kontext");
        }
    };
    let get_context = async function(role, permission){
        const res = $.ajax({
            type: "GET",
            url: "/api/roles/" + role + "/permissions/" + permission + "/context",
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    }
    return {
        load_templates: load_templates,
        load_role: load_role,
        permission_checkbox_toggle: permission_checkbox_toggle,
    }
}());