use bigdecimal::BigDecimal;
use diesel::{ExpressionMethods, QueryDsl, RunQueryDsl};
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::Settings;

#[derive(Queryable, Clone)]
pub struct RawBillingRate {
    pub(crate) billing_rate_id: uuid::Uuid,
    pub(crate) name: String,
    pub(crate) description: Option<String>,
    pub(crate) active: bool,
    pub(crate) payment_per_hour: BigDecimal,
    pub(crate) lump_sum: BigDecimal,
}

pub fn get_billing_rates(settings: &State<Settings>) -> Result<Vec<RawBillingRate>, diesel::result::Error> {
    use crate::schema::personnel_billing_rates::dsl::*;

    let connection = establish_connection(settings);

    match personnel_billing_rates.load(&connection) {
        Ok(rates) => Ok(rates),
        Err(e) => {
            error!("Couldn't retrieve billing rates: {}", e);
            Err(e)
        }
    }
}

pub fn get_billing_rate(settings: &State<Settings>, billing_rate_id: uuid::Uuid) -> Result<RawBillingRate, diesel::result::Error> {
    use crate::schema::personnel_billing_rates;

    let connection = establish_connection(settings);

    match personnel_billing_rates::table.filter(personnel_billing_rates::dsl::billing_rate_id.eq(billing_rate_id)).get_result(&connection) {
        Ok(rate) => Ok(rate),
        Err(e) => {
            error!("Couldn't retrieve billing rate: {}", e);
            Err(e)
        }
    }
}