use rocket::State;
use crate::helper::settings::Settings;
use crate::modules::member_management::model::member::CreateMemberForm;
use crate::database::controller::connector::establish_connection;

use diesel::{RunQueryDsl};


use crate::database::model::members::CreateMember;
use crate::helper::forms::into_option;
use chrono::NaiveDate;
use crate::database::controller::members_groups::add_member_to_group;

use crate::database::controller::entities::generate_entity;
use crate::database::controller::roles::add_member_to_role;

pub fn create_member(settings: &State<Settings>, form: CreateMemberForm) -> Result<uuid::Uuid, diesel::result::Error>{
    let connection = establish_connection(settings);

    let entity_id2 = generate_entity(settings)?;

    use crate::schema::members::dsl::*;

    let personnel_number2 = if form.personnel_number.is_empty(){
        None
    }else{
        match form.personnel_number.parse::<i32>(){
            Ok(num) => Some(num),
            Err(e) => {
                warn!("Couldn't parse personnel number: {}", e);
                None
            }
        }
    };

    let entrance_date2 = if form.entrance_date.is_empty(){
        None
    }else {
        match NaiveDate::parse_from_str(&form.entrance_date, "%Y-%m-%d") {
            Ok(date) => Some(date),
            Err(e) => {
                warn!("Couldn't parse date: {}", e);
                None
            }
        }
    };

    let member = CreateMember{
        entity_id: entity_id2,
        firstname: form.firstname,
        lastname: form.lastname,
        sex: form.sex.parse::<i16>().unwrap(),
        salutation: form.salutation,
        academic_titles: into_option(form.academic_titles),
        personnel_number: personnel_number2,
        entrance_date: entrance_date2
    };

    match add_member_to_group(settings, entity_id2, uuid::Uuid::parse_str("00000000-0000-0000-0000-000000000000").unwrap()){
        Ok(_) => {},
        Err(_) => {}
    }

    match uuid::Uuid::parse_str(&form.group){
        Ok(group) => match add_member_to_group(settings,entity_id2, group){
            Ok(_) => (),
            Err(e) => {
                error!("Couldn't add member to group: {}", e);
                return Err(e)
            }
        },
        Err(e) => {
            error!("Couldn't parse group_id for new member: {}", e);
        }
    }

    if settings.application.new_member_default_role != "" {
        add_member_to_role(settings, entity_id2, settings.application.new_member_default_role.clone());
    }

    match diesel::insert_into(members).values(member).execute(&connection){
        Ok(_) => Ok(entity_id2),
        Err(e) => {
            error!("Couldn't create member: {}", e);
            Err(e)
        }
    }

}