use diesel::{ExpressionMethods, RunQueryDsl};
use diesel::query_dsl::filter_dsl::FilterDsl;
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::helper::settings::Settings;
use crate::schema::entities::dsl::entities;

pub fn generate_entity(settings: &State<Settings>) -> Result<uuid::Uuid, diesel::result::Error> {
    let connection = establish_connection(settings);

    match diesel::insert_into(entities).default_values().returning(crate::schema::entities::dsl::entity_id).get_result(&connection){
        Ok(entity_id) => Ok(entity_id),
        Err(e) => {
            error!("Couldn't create new entity: {}", e);
            Err(e)
        }
    }
}

/// Deprecated!
/// Security Issue: This method doesn't check the entity type. This allows users to delete any resource
// TODO: remove
#[deprecated]
pub fn remove_entity(settings: &State<Settings>, entity_id: uuid::Uuid) -> Result<(), diesel::result::Error> {
    let connection = establish_connection(settings);

    match diesel::delete(entities.filter(crate::schema::entities::dsl::entity_id.eq(entity_id))).execute(&connection) {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't delete entity_id: {}", e);
            Err(e)
        }
    }
}