use crate::helper::settings::Settings;
use rocket::State;
use crate::database::controller::connector::establish_connection;
use diesel::{RunQueryDsl, ExpressionMethods};
use crate::database::model::units::RawUnit;
use diesel::query_dsl::filter_dsl::FilterDsl;

pub fn get_units(settings: &State<Settings>) -> Result<Vec<RawUnit>, diesel::result::Error>{
    use crate::schema::units::dsl::*;

    let connection = establish_connection(settings);
    match units.load::<RawUnit>(&connection){
        Ok(unit_list) => Ok(unit_list),
        Err(e) => {
            error!("Couldn't load unit list: {}", e);
            Err(e)
        }
    }
}

pub fn get_unit(settings: &State<Settings>, unit_id2: uuid::Uuid) -> Result<RawUnit, diesel::result::Error>{
    use crate::schema::units::dsl::*;

    let connection = establish_connection(settings);
    match units.filter(unit_id.eq(unit_id2)).first(&connection){
        Ok(unit) => Ok(unit),
        Err(e) => {
            error!("Couldn't load unit: {}", e);
            Err(e)
        }
    }
}