use crate::database::controller::members::check_access_to_resource;
use crate::helper::settings::Settings;
use crate::modules::member_management::model::groups::Group;
use crate::modules::member_management::model::member::Member;
use rocket::State;

//TODO: REMOVE LEGACY CHECK ACCESS
pub fn check_access_legacy(
    settings: &State<Settings>,
    member_to_check: &Member,
    own_member_id: uuid::Uuid,
    permission: String,
) -> bool {
    let mut access = false;

    for group in member_to_check.groups.iter() {
        if check_access_to_resource(
            settings,
            own_member_id,
            group.entity_id,
            &permission.clone(),
        ) {
            trace!(
                "Access for member {} to group {} for permission {}!",
                own_member_id,
                group.entity_id,
                permission.clone()
            );
            access = true;
            break;
        } else {
            trace!(
                "No access for member {} to group {} for permission {}!",
                own_member_id,
                group.entity_id,
                permission.clone()
            );
        }
    }

    //Check if any role has direct access to member
    if !access {
        if check_access_to_resource(
            settings,
            own_member_id,
            member_to_check.entity_id,
            &permission,
        ) {
            access = true;
        }
    }
    access
}

/// Checks if caller has access to a specific member's group or member directly
pub fn check_access_to_member_or_group(
    settings: &State<Settings>,
    member_to_check_id: uuid::Uuid,
    member_to_check_groups: Vec<Group>,
    caller_entity_id: uuid::Uuid,
    permission: String,
) -> bool {
    let mut access = false;

    for group in member_to_check_groups {
        if check_access_to_resource(
            settings,
            caller_entity_id,
            group.entity_id,
            &permission.clone(),
        ) {
            trace!(
                "Access for member {} to group {}!",
                caller_entity_id,
                group.entity_id
            );
            access = true;
            break;
        } else {
            trace!(
                "No access for member {} to group {}!",
                caller_entity_id,
                group.entity_id
            );
        }
    }

    //Check if any role has direct access to member
    if !access {
        if check_access_to_resource(settings, caller_entity_id, member_to_check_id, &permission) {
            access = true;
        }
    }
    access
}
