use crate::helper::session_cookies::model::SessionCookie;
use rocket::State;
use crate::helper::settings::Settings;

use rocket::http::Status;
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::helper::sitebuilder::model::general::{Header, Stylesheet, Footer, Script};
use rocket_dyn_templates::Template;
use crate::database::model::roles::Role;
use crate::database::controller::roles::get_roles;

#[derive(Serialize)]
pub struct SettingsModule {
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
    pub caller: uuid::Uuid,
    pub roles: Vec<Role>
}

#[get("/portal/settings/permissions")]
pub fn settings_permissions(cookie: SessionCookie, settings: &State<Settings>) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::events::VIEW.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Einstellungen".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/settings_permissions.js".to_string(),
        }, Script {
            path: "/js/mini_searchbar.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.settings.active = true;

    let roles = match get_roles(settings){
        Ok(roles) => roles,
        Err(e) => {error!("Couldn't get roles: {}", e);return Err(Status::InternalServerError)}
    };
    
    let module = SettingsModule {
        header,
        footer,
        sidebar,
        caller: member.entity_id,
        roles
    };

    Ok(Template::render("module_settings_permissions", module))
}