use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::billing::states::{BillingState, get_billing_states};
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::Settings;

#[get("/api/billing_states", format = "json")]
pub fn read_billing_states(settings: &State<Settings>, cookie: SessionCookie) -> Result<Json<Vec<BillingState>>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    //TODO: add overwrite if member_responsible
    if !caller.has_permission(crate::permissions::modules::event_billing::VIEW.to_string()) {
        return Err(Json(ApiError::new(403, "Keine Berechtigung Abrechnungsstatusse abzurufen!".to_string()).to_wrapper()));
    }

    match get_billing_states(settings) {
        Ok(states) => Ok(Json(states)),
        Err(e) => Err(translate_diesel(e)),
    }
}