use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::database::model::organisers::Organiser;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::database::controller::entities::generate_entity;
use crate::database::controller::organisers::add_organiser;
use crate::helper::translate_diesel_error::translate_diesel;

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct CreateOrganiserData{
    pub(crate) company: Option<String>,
    pub(crate) firstname: Option<String>,
    pub(crate) lastname: Option<String>,
    pub(crate) phone: Option<String>,
    pub(crate) email: Option<String>,
    pub(crate) other: Option<String>,
}

#[post("/api/event_organisers", format = "json", data = "<create_organiser_data>")]
pub fn create_organiser(
    settings: &State<Settings>,
    cookie: SessionCookie,
    create_organiser_data: Json<CreateOrganiserData>,
) -> Result<Json<Organiser>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::organisers::CREATE.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, neuen Veranstalter anzulegen!".to_string()).to_wrapper(),
        ));
    }

    let organiser_create_data = create_organiser_data.into_inner();

    let entity_id2 = match generate_entity(settings){
        Ok(ent) => ent,
        Err(_e) => return Err(Json(ApiError::new(500, "Konnte keine neue Entität anlegen!".to_string()).to_wrapper()))
    };

    let input: Organiser = Organiser{
        entity_id: entity_id2,
        company: organiser_create_data.company,
        firstname: organiser_create_data.firstname,
        lastname: organiser_create_data.lastname,
        phone: organiser_create_data.phone,
        email: organiser_create_data.email,
        other: organiser_create_data.other
    };

    match add_organiser(settings, input){
        Ok(org) => Ok(Json(org)),
        Err(e) => Err(translate_diesel(e))
    }
}