use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use crate::modules::api::member_management::controller::parser::parse_member_cookie;

use crate::helper::translate_diesel_error::translate_diesel;

use crate::database::controller::entities::remove_entity;

#[delete("/api/event_organisers", format = "json", data = "<organiser_list>")]
pub fn delete_organisers(
    settings: &State<Settings>,
    cookie: SessionCookie,
    organiser_list: Json<Vec<uuid::Uuid>>,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::organisers::DELETE.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, Veranstalter zu löschen!".to_string()).to_wrapper(),
        ));
    }

    let organiser_list = organiser_list.into_inner();

    for organiser in organiser_list {
        match remove_entity(settings, organiser){
                Ok(_) => {}
                Err(e) => return Err(translate_diesel(e)),
        }
    }

    Ok(())
}