use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::event_organisers::create::CreateOrganiserData;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::database::model::organisers::Organiser;
use crate::database::controller::organisers::change_organiser;
use crate::helper::translate_diesel_error::translate_diesel;

#[put("/api/event_organisers/<organiser_id>", format = "json", data = "<change_organiser_data>")]
pub fn update_organiser(
    settings: &State<Settings>,
    cookie: SessionCookie,
    change_organiser_data: Json<CreateOrganiserData>,
    organiser_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.clone().member)?;
    let organiser_id = parse_uuid_string(organiser_id)?;

    if !caller.has_permission(crate::permissions::modules::event_management::organisers::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, Veranstalter zu bearbeiten!".to_string()).to_wrapper(),
        ));
    }

    let cod = change_organiser_data.into_inner();

    let org : Organiser = Organiser{
        entity_id: organiser_id,
        company: cod.company,
        firstname: cod.firstname,
        lastname: cod.lastname,
        phone: cod.phone,
        email: cod.email,
        other: cod.other
    };

    match change_organiser(settings, org){
        Ok(_) => Ok(()),
        Err(e) => Err(translate_diesel(e))
    }

}