use rocket::serde::json::Json;
use rocket::State;

use crate::database::model::permissions::Permission;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

/// Get all permissions
///
/// # Api Call
/// * GET
/// * /api/permissions/
///
/// # Api Result
/// * Result JSON<Vec<[Permission]>> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.view
#[get("/api/permissions", format = "json")]
pub fn get_permissions(
    settings: &State<Settings>,
    cookie: SessionCookie,
) -> Result<Json<Vec<Permission>>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::VIEW.to_string()){
        return Err(Json(ApiError::new(403, "No permission to read permission list".to_string()).to_wrapper()))
    }

    match crate::database::controller::permissions::get_permissions(settings) {
        Ok(res) => Ok(Json(res)),
        Err(e) => Err(translate_diesel(e))
    }
}