use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::model::api_outcome::ApiErrorWrapper;
use crate::database::model::units::RawUnit;
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::database::controller::members::check_access_to_resource;
use crate::database::controller::units::get_units;
use crate::helper::translate_diesel_error::translate_diesel;

#[get("/api/units?<with_caller_permission>", format = "json")]
pub fn read_unit_list(
    settings: &State<Settings>,
    cookie: SessionCookie,
    with_caller_permission: Option<String>
) -> Result<Json<Vec<RawUnit>>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    let units = match get_units(settings){
        Ok(units) => units,
        Err(e) => return Err(translate_diesel(e))
    };

    match with_caller_permission{
        None => Ok(Json(units)),
        Some(permission) => {
            let mut unit_list : Vec<RawUnit> = vec![];
            for unit in units{
                if check_access_to_resource(settings, caller.entity_id, unit.unit_id, &permission){
                    unit_list.push(unit);
                }
            }
            Ok(Json(unit_list))
        }
    }
}