use crate::helper::sitebuilder::model::alerts::Alert;
use crate::helper::sitebuilder::model::general::{Footer, Header, Stylesheet, Script};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::dashboard::model::dashboard_module::DashboardModule;
use crate::modules::member_management::model::member::Member;
use rocket::State;
use crate::helper::settings::Settings;

pub fn get_context(settings: &State<Settings>, member: Member, alert: Option<Alert>) -> DashboardModule {
    let header = Header {
        html_language: "de".to_string(),
        site_title: "Dashboard".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer { scripts: vec![Script{
        path: "/js/dashboard.js".to_string()
    }
    ] };

    let mut sidebar = Sidebar::new(member.clone());
    sidebar.summary.active = true;

    DashboardModule {
        header,
        footer,
        sidebar,
        alert,
        user_support_email: settings.application.user_support_email.clone(),
        caller: member.entity_id
    }
}
