use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;

use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::api::member_management::controller::parser::parse_uuid_string;

#[derive(Serialize)]
pub struct EditEvent {
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
    pub caller: uuid::Uuid,
    pub event_id: uuid::Uuid,
}

#[get("/portal/em/event?<id>")]
pub fn edit_event(cookie: SessionCookie, _settings: &State<Settings>, id: String) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::events::EDIT.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Einsatz bearbeiten".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![
            Script {
                path: "/js/mini_searchbar.js".to_string(),
            },
            Script {
                path: "/js/em_edit_event.js".to_string(),
            },
            Script {
                path: "/js/search2.js".to_string(),
            }],
    };

    let mut sidebar = Sidebar::new(member.clone());
    sidebar.event_management.active = true;

    let eventlist = EditEvent {
        header,
        footer,
        sidebar,
        caller: member.entity_id,
        event_id: parse_uuid_string(id).unwrap(), //TODO: do not unwrap
    };

    Ok(Template::render("module_em_edit_event", eventlist))
}
