use crate::helper::session_cookies::model::SessionCookie;

use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use rocket::http::Status;
use rocket_dyn_templates::Template;


#[derive(Serialize)]
pub struct Organisers {
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
}
#[get("/portal/em/organisers")]
pub fn organisers(cookie: SessionCookie) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::organisers::VIEW.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Veranstalter".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/em_organisers.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.event_management.active = true;

    let organisers = Organisers {
        header,
        footer,
        sidebar,
    };

    Ok(Template::render("module_em_organisers", organisers))
}
