use rocket::http::Status;
use crate::modules::member_management::model::member_module::MemberModuleCreate;
use crate::helper::sitebuilder::model::general::{Header, Footer, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::member_management::model::member::Member;
use crate::database::controller::groups::get_raw_groups;
use crate::helper::settings::Settings;
use rocket::State;

pub fn render(caller: Member, settings: &State<Settings>) -> Result<MemberModuleCreate, Status>{
    let header = Header {
        html_language: "de".to_string(),
        site_title: "Mitglied erstellen".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer{
        scripts: vec![]
    };
    let sidebar = Sidebar::new(caller);

    let groups = match get_raw_groups(settings){
        Ok(groups) => groups,
        Err(_) => vec![]
    };

    Ok(MemberModuleCreate{
        header,
        footer,
        sidebar,
        groups
    })
}